@@ -19,7 +19,10 @@
 
 *****************************************************************************/
 
+#include "synthv1_config.h"
+
 #include "synthv1_jack.h"
+#include "synthv1_param.h"
 
 #include <jack/midiport.h>
 
@@ -28,6 +31,9 @@
 
 #include <math.h>
 
+#include <QCoreApplication>
+#include <QDir>
+
 
 #ifdef CONFIG_ALSA_MIDI
 
@@ -106,6 +112,28 @@ int synthv1_jack_process ( jack_nframes_t nframes, void *arg )
 
 
 
+#ifdef CONFIG_JACK_SESSION
+
+#include <jack/session.h>
+
+#include <QDir>
+
+//----------------------------------------------------------------------
+// synthv1_jack_session_event -- JACK session event callabck
+//
+
+static void synthv1_jack_session_event (
+	jack_session_event_t *pSessionEvent, void *pvArg )
+{
+	synthv1_jack *pSynth = static_cast<synthv1_jack *> (pvArg);
+
+	if (pSynth)
+		pSynth->sessionEvent(pSessionEvent);
+}
+
+#endif	// CONFIG_JACK_SESSION
+
+
 //-------------------------------------------------------------------------
 // synthv1_jack - impl.
 //
@@ -137,8 +165,8 @@ synthv1_jack::synthv1_jack (void) : synthv1(2)
 
 	m_bpm = 0.0f;
 
-//	open(SYNTHV1_TITLE);
-//	activate();
+	open(SYNTHV1_TITLE);
+	activate();
 }
 
 
@@ -245,8 +273,11 @@ int synthv1_jack::process ( jack_nframes_t nframes )
 void synthv1_jack::open ( const char *client_id )
 {
 	// init param ports
-	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i)
-		synthv1::setParamPort(synthv1::ParamIndex(i), &m_params[i]);
+	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i) {
+		synthv1::ParamIndex index = synthv1::ParamIndex(i);
+		m_params[i] = synthv1_param::paramDefaultValue(index);
+		synthv1::setParamPort(index, &m_params[i]);
+	}
 
 	// open client
 	m_client = ::jack_client_open(client_id, JackNullOption, NULL);
@@ -308,6 +339,14 @@ void synthv1_jack::open ( const char *client_id )
 	// set process callbacks...
 	::jack_set_process_callback(m_client,
 		synthv1_jack_process, this);
+
+#ifdef CONFIG_JACK_SESSION
+	// JACK session event callback...
+	if (::jack_set_session_callback) {
+		::jack_set_session_callback(m_client,
+			synthv1_jack_session_event, this);
+	}
+#endif
 }
 
 
@@ -509,4 +548,323 @@ void synthv1_jack::alsa_capture ( snd_seq_event_t *ev )
 #endif	// CONFIG_ALSA_MIDI
 
 
+#ifdef CONFIG_JACK_SESSION
+
+// JACK session event handler.
+void synthv1_jack::sessionEvent ( void *pvSessionArg )
+{
+	jack_session_event_t *pJackSessionEvent
+		= (jack_session_event_t *) pvSessionArg;
+
+#ifdef CONFIG_DEBUG
+	qDebug("synthv1_jack::sessionEvent()"
+		" type=%d client_uuid=\"%s\" session_dir=\"%s\"",
+		int(pJackSessionEvent->type),
+		pJackSessionEvent->client_uuid,
+		pJackSessionEvent->session_dir);
+#endif
+
+	const bool bQuit = (pJackSessionEvent->type == JackSessionSaveAndQuit);
+
+	const QString sSessionDir
+		= QString::fromUtf8(pJackSessionEvent->session_dir);
+	const QString sSessionName
+		= QFileInfo(QFileInfo(sSessionDir).canonicalPath()).completeBaseName();
+	const QString sSessionFile = sSessionName + '.' + SYNTHV1_TITLE;
+
+	QStringList args;
+	args << QCoreApplication::applicationFilePath();
+	args << QString("\"${SESSION_DIR}%1\"").arg(sSessionFile);
+
+	synthv1_param::savePreset(this,
+		QFileInfo(sSessionDir, sSessionFile).absoluteFilePath());
+
+	const QByteArray aCmdLine = args.join(" ").toUtf8();
+	pJackSessionEvent->command_line = ::strdup(aCmdLine.constData());
+
+	jack_session_reply(m_client, pJackSessionEvent);
+	jack_session_event_free(pJackSessionEvent);
+
+	if (bQuit)
+		QCoreApplication::quit();
+}
+
+#endif	// CONFIG_JACK_SESSION
+
+
+//-------------------------------------------------------------------------
+// synthv1_application -- Singleton application instance.
+//
+
+#include "synthv1widget_jack.h"
+
+#include <QApplication>
+#include <QTextStream>
+
+#ifdef CONFIG_NSM
+#include "synthv1_nsm.h"
+#endif
+
+
+// Constructor.
+synthv1_application::synthv1_application ( int& argc, char **argv )
+	: QObject(NULL), m_pApp(NULL), m_bGui(true),
+		m_pSynth(NULL), m_pWidget(NULL)
+	  #ifdef CONFIG_NSM
+		, m_pNsmClient(NULL)
+	  #endif
+{
+#ifdef Q_WS_X11
+	m_bGui = (::getenv("DISPLAY") != 0);
+#endif
+	for (int i = 1; i < argc; ++i) {
+		const QString& sArg = QString::fromLocal8Bit(argv[i]);
+		if (sArg[0] != '-')
+			m_presets.append(sArg);
+		else
+		if (sArg == "-g" || sArg == "--no-gui")
+			m_bGui = false;
+	}
+
+	if (m_bGui)
+		m_pApp = new QApplication(argc, argv);
+	else
+		m_pApp = new QCoreApplication(argc, argv);
+}
+
+
+// Destructor.
+synthv1_application::~synthv1_application (void)
+{
+#ifdef CONFIG_NSM
+	if (m_pNsmClient) delete m_pNsmClient;
+#endif
+	if (m_pWidget) delete m_pWidget;
+	if (m_pSynth) delete m_pSynth;
+	if (m_pApp) delete m_pApp;
+}
+
+
+// Argument parser method.
+bool synthv1_application::parse_args (void)
+{
+	QTextStream out(stderr);
+
+	const QStringList& args = m_pApp->arguments();
+	QStringListIterator iter(args);
+	while (iter.hasNext()) {
+		const QString& sArg = iter.next();
+		if (sArg == "-h" || sArg == "--help") {
+			out << QObject::tr(
+				"Usage: %1 [options] [preset-file]\n\n"
+				SYNTHV1_TITLE " - " SYNTHV1_SUBTITLE "\n\n"
+				"Options:\n\n"
+				"  -g, --no-gui\n\tDisable the graphical user interface (GUI)\n\n"
+				"  -h, --help\n\tShow help about command line options\n\n"
+				"  -v, --version\n\tShow version information\n\n")
+				.arg(args.at(0));
+			return false;
+		}
+		else
+		if (sArg == "-v" || sArg == "-V" || sArg == "--version") {
+			out << QObject::tr("Qt: %1\n").arg(qVersion());
+			out << QObject::tr(SYNTHV1_TITLE ": %1\n").arg(SYNTHV1_VERSION);
+			return false;
+		}
+	}
+
+	return true;
+}
+
+
+// Startup methods.
+bool synthv1_application::setup (void)
+{
+	if (m_pApp == NULL)
+		return false;
+
+	if (!parse_args()) {
+		m_pApp->quit();
+		return false;
+	}
+
+	m_pSynth = new synthv1_jack();
+
+	if (m_bGui) {
+		m_pWidget = new synthv1widget_jack(m_pSynth);
+		if (m_presets.isEmpty())
+			m_pWidget->initPreset();
+		else
+			m_pWidget->loadPreset(m_presets.first());
+		m_pWidget->show();
+	}
+	else
+	if (!m_presets.isEmpty())
+		synthv1_param::loadPreset(m_pSynth, m_presets.first());
+
+#ifdef CONFIG_NSM
+	// Check whether to participate into a NSM session...
+	const QString& nsm_url
+		= QString::fromLatin1(::getenv("NSM_URL"));
+	if (!nsm_url.isEmpty()) {
+		m_pNsmClient = new synthv1_nsm(nsm_url);
+		QObject::connect(m_pNsmClient,
+			SIGNAL(open()),
+			SLOT(openSession()));
+		QObject::connect(m_pNsmClient,
+			SIGNAL(save()),
+			SLOT(saveSession()));
+		QObject::connect(m_pNsmClient,
+			SIGNAL(show()),
+			SLOT(showSession()));
+		QObject::connect(m_pNsmClient,
+			SIGNAL(hide()),
+			SLOT(hideSession()));
+		QString caps(":switch:dirty:");
+		if (m_bGui)
+			caps += "optional-gui:";
+		m_pNsmClient->announce(SYNTHV1_TITLE, caps.toLatin1().constData());
+		if (m_pWidget)
+			m_pWidget->setNsmClient(m_pNsmClient);
+	}
+#endif	// CONFIG_NSM
+
+	return true;
+}
+
+
+// Facade method.
+int synthv1_application::exec (void)
+{
+	return (setup() ? m_pApp->exec() : 1);
+}
+
+
+#ifdef CONFIG_NSM
+
+void synthv1_application::openSession (void)
+{
+	if (m_pSynth == NULL)
+		return;
+
+	if (m_pNsmClient == NULL)
+		return;
+
+	if (!m_pNsmClient->is_active())
+		return;
+
+#ifdef CONFIG_DEBUG
+	qDebug("synthv1_jack::openSession()");
+#endif
+
+	m_pSynth->deactivate();
+	m_pSynth->close();
+
+	const QString& client_id = m_pNsmClient->client_id();
+	const QString& path_name = m_pNsmClient->path_name();
+	const QString& display_name = m_pNsmClient->display_name();
+
+	m_pSynth->open(client_id.toUtf8().constData());
+	m_pSynth->activate();
+
+	const QDir dir(path_name);
+	if (!dir.exists())
+		dir.mkpath(path_name);
+
+	const QFileInfo fi(path_name, display_name + '.' + SYNTHV1_TITLE);
+	if (fi.exists()) {
+		const QString& sFilename = fi.absoluteFilePath();
+		if (m_pWidget) {
+			m_pWidget->loadPreset(sFilename);
+		} else {
+			synthv1_param::loadPreset(m_pSynth, sFilename);
+		}
+	}
+
+	m_pNsmClient->open_reply();
+	m_pNsmClient->dirty(false);
+
+	if (m_pWidget)
+		m_pNsmClient->visible(m_pWidget->isVisible());
+}
+
+void synthv1_application::saveSession (void)
+{
+	if (m_pSynth == NULL)
+		return;
+
+	if (m_pNsmClient == NULL)
+		return;
+
+	if (!m_pNsmClient->is_active())
+		return;
+
+#ifdef CONFIG_DEBUG
+	qDebug("synthv1_jack::saveSession()");
+#endif
+
+	const QString& path_name = m_pNsmClient->path_name();
+	const QString& display_name = m_pNsmClient->display_name();
+//	const QString& client_id = m_pNsmClient->client_id();
+	const QFileInfo fi(path_name, display_name + '.' + SYNTHV1_TITLE);
+
+	synthv1_param::savePreset(m_pSynth, fi.absoluteFilePath());
+
+	m_pNsmClient->save_reply();
+	m_pNsmClient->dirty(false);
+}
+
+
+void synthv1_application::showSession (void)
+{
+	if (m_pNsmClient == NULL)
+		return;
+
+	if (!m_pNsmClient->is_active())
+		return;
+
+#ifdef CONFIG_DEBUG
+	qDebug("synthv1_jack::showSession()");
+#endif
+
+	if (m_pWidget) {
+		m_pWidget->show();
+		m_pWidget->raise();
+		m_pWidget->activateWindow();
+	}
+}
+
+void synthv1_application::hideSession (void)
+{
+	if (m_pNsmClient == NULL)
+		return;
+
+	if (!m_pNsmClient->is_active())
+		return;
+
+#ifdef CONFIG_DEBUG
+	qDebug("synthv1_jack::hideSession()");
+#endif
+
+	if (m_pWidget)
+		m_pWidget->hide();
+}
+
+
+#endif	// CONFIG_NSM
+
+
+//-------------------------------------------------------------------------
+// main
+
+int main ( int argc, char *argv[] )
+{
+	Q_INIT_RESOURCE(synthv1);
+
+	synthv1_application app(argc, argv);
+
+	return app.exec();
+}
+
+
 // end of synthv1_jack.cpp
