@@ -22,9 +22,9 @@
 #include "synthv1widget.h"
 #include "synthv1_param.h"
 
-#include <QDomDocument>
-#include <QTextStream>
-#include <QFileInfo>
+#include "synthv1_wave.h"
+
+#include "synthv1widget_config.h"
 
 #include <QMessageBox>
 #include <QDir>
@@ -84,6 +84,7 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	shapes << tr("Pulse");
 	shapes << tr("Saw");
 	shapes << tr("Sine");
+	shapes << tr("Rand");
 	shapes << tr("Noise");
 
 	m_ui.Dco1Shape1Knob->insertItems(0, shapes);
@@ -705,7 +706,14 @@ synthv1widget::synthv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	QObject::connect(m_ui.TabBar, SIGNAL(currentChanged(int)),
 		m_ui.StackedWidget, SLOT(setCurrentIndex(int)));
 
+	synthv1widget_config *pConfig = synthv1widget_config::getInstance();
+	if (pConfig)
+		m_ui.helpUseNativeDialogsAction->setChecked(pConfig->bUseNativeDialogs);
+
 	// Menu actions
+	QObject::connect(m_ui.helpUseNativeDialogsAction,
+		SIGNAL(triggered(bool)),
+		SLOT(helpUseNativeDialogs(bool)));
 	QObject::connect(m_ui.helpAboutAction,
 		SIGNAL(triggered(bool)),
 		SLOT(helpAbout()));
@@ -802,6 +810,22 @@ void synthv1widget::updateParamEx ( synthv1::ParamIndex index, float fValue )
 	++m_iUpdate;
 
 	switch (index) {
+	case synthv1::DCO1_SHAPE1:
+		m_ui.Dco1Bandl1Knob->setEnabled(
+			synthv1_wave::Shape(int(fValue)) != synthv1_wave::Noise);
+		break;
+	case synthv1::DCO1_SHAPE2:
+		m_ui.Dco1Bandl2Knob->setEnabled(
+			synthv1_wave::Shape(int(fValue)) != synthv1_wave::Noise);
+		break;
+	case synthv1::DCO2_SHAPE1:
+		m_ui.Dco2Bandl1Knob->setEnabled(
+			synthv1_wave::Shape(int(fValue)) != synthv1_wave::Noise);
+		break;
+	case synthv1::DCO2_SHAPE2:
+		m_ui.Dco2Bandl2Knob->setEnabled(
+			synthv1_wave::Shape(int(fValue)) != synthv1_wave::Noise);
+		break;
 	case synthv1::DEL1_BPMSYNC:
 		if (fValue > 0.0f)
 			m_ui.Del1BpmKnob->setValue(0.0f);
@@ -879,6 +903,28 @@ void synthv1widget::resetSwapParams (void)
 }
 
 
+// Initialize param values.
+void synthv1widget::updateParamValues (void)
+{
+	resetSwapParams();
+
+	synthv1 *pSynth = instance();
+
+	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i) {
+		synthv1::ParamIndex index = synthv1::ParamIndex(i);
+		float fValue = synthv1_param::paramDefaultValue(index);
+		const float *pfParamPort
+			= (pSynth ? pSynth->paramPort(index) : NULL);
+		if (pfParamPort)
+			fValue = *pfParamPort;
+		setParamValue(index, fValue, true);
+		updateParam(index, fValue);
+		updateParamEx(index, fValue);
+		m_params_ab[index] = fValue;
+	}
+}
+
+
 // Reset all param default values.
 void synthv1widget::resetParamValues (void)
 {
@@ -945,78 +991,19 @@ void synthv1widget::loadPreset ( const QString& sFilename )
 	if (pSynth == NULL)
 		return;
 
-	QFile file(sFilename);
-	if (!file.open(QIODevice::ReadOnly))
-		return;
-
-	static QHash<QString, synthv1::ParamIndex> s_hash;
-	if (s_hash.isEmpty()) {
-		for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i) {
-			synthv1::ParamIndex index = synthv1::ParamIndex(i);
-			s_hash.insert(synthv1_param::paramName(index), index);
-		}
-	}
-
 	resetParamKnobs();
 	resetParamValues();
 
-	pSynth->reset();
+	synthv1_param::loadPreset(pSynth, sFilename);
 
-	const QFileInfo fi(sFilename);
-	const QDir currentDir(QDir::current());
-	QDir::setCurrent(fi.absolutePath());
+	updateParamValues();
 
-	QDomDocument doc(SYNTHV1_TITLE);
-	if (doc.setContent(&file)) {
-		QDomElement ePreset = doc.documentElement();
-		if (ePreset.tagName() == "preset"
-			&& ePreset.attribute("name") == fi.completeBaseName()) {
-			for (QDomNode nChild = ePreset.firstChild();
-					!nChild.isNull();
-						nChild = nChild.nextSibling()) {
-				QDomElement eChild = nChild.toElement();
-				if (eChild.isNull())
-					continue;
-				if (eChild.tagName() == "params") {
-					for (QDomNode nParam = eChild.firstChild();
-							!nParam.isNull();
-								nParam = nParam.nextSibling()) {
-						QDomElement eParam = nParam.toElement();
-						if (eParam.isNull())
-							continue;
-						if (eParam.tagName() == "param") {
-							synthv1::ParamIndex index = synthv1::ParamIndex(
-								eParam.attribute("index").toULong());
-							const QString& sName = eParam.attribute("name");
-							if (!sName.isEmpty()) {
-								if (!s_hash.contains(sName))
-									continue;
-								index = s_hash.value(sName);
-							}
-							float fValue = eParam.text().toFloat();
-						//--legacy support < 0.3.0.4 -- begin
-							if (index == synthv1::DEL1_BPM && fValue < 3.6f)
-								fValue *= 100.0f;
-						//--legacy support < 0.3.0.4 -- end.
-							setParamValue(index, fValue, true);
-							updateParam(index, fValue);
-							m_params_ab[index] = fValue;
-						}
-					}
-				}
-			}
-		}
-	}
+	const QString& sPreset
+		= QFileInfo(sFilename).completeBaseName();
 
-	file.close();
-
-	const QString& sPreset = fi.completeBaseName();
 	m_ui.Preset->setPreset(sPreset);
-
 	m_ui.StatusBar->showMessage(tr("Load preset: %1").arg(sPreset), 5000);
 	updateDirtyPreset(false);
-
-	QDir::setCurrent(currentDir.absolutePath());
 }
 
 
@@ -1025,31 +1012,11 @@ void synthv1widget::savePreset ( const QString& sFilename )
 #ifdef CONFIG_DEBUG
 	qDebug("synthv1widget::savePreset(\"%s\")", sFilename.toUtf8().constData());
 #endif
-	const QString& sPreset = QFileInfo(sFilename).completeBaseName();
 
-	QDomDocument doc(SYNTHV1_TITLE);
-	QDomElement ePreset = doc.createElement("preset");
-	ePreset.setAttribute("name", sPreset);
-	ePreset.setAttribute("version", SYNTHV1_VERSION);
+	synthv1_param::savePreset(instance(), sFilename);
 
-	QDomElement eParams = doc.createElement("params");
-	for (uint32_t i = 0; i < synthv1::NUM_PARAMS; ++i) {
-		QDomElement eParam = doc.createElement("param");
-		synthv1::ParamIndex index = synthv1::ParamIndex(i);
-		eParam.setAttribute("index", QString::number(i));
-		eParam.setAttribute("name", synthv1_param::paramName(index));
-		eParam.appendChild(
-			doc.createTextNode(QString::number(paramValue(index))));
-		eParams.appendChild(eParam);
-	}
-	ePreset.appendChild(eParams);
-	doc.appendChild(ePreset);
-
-	QFile file(sFilename);
-	if (file.open(QIODevice::WriteOnly | QIODevice::Truncate)) {
-		QTextStream(&file) << doc.toString();
-		file.close();
-	}
+	const QString& sPreset
+		= QFileInfo(sFilename).completeBaseName();
 
 	m_ui.StatusBar->showMessage(tr("Save preset: %1").arg(sPreset), 5000);
 	updateDirtyPreset(false);
@@ -1064,6 +1031,16 @@ bool synthv1widget::queryClose (void)
 
 
 // Menu actions.
+void synthv1widget::helpUseNativeDialogs ( bool bOn )
+{
+	synthv1widget_config *pConfig = synthv1widget_config::getInstance();
+	if (pConfig) {
+		pConfig->bUseNativeDialogs = bOn;
+		pConfig->bDontUseNativeDialogs = !pConfig->bUseNativeDialogs;
+	}
+}
+
+
 void synthv1widget::helpAbout (void)
 {
 	// About...
@@ -1112,6 +1089,7 @@ void synthv1widget::helpAbout (void)
 	QMessageBox::about(this, tr("About") + " " SYNTHV1_TITLE, sText);
 }
 
+
 void synthv1widget::helpAboutQt (void)
 {
 	// About Qt...
